import math

class Point:
	def __init__(self, x, y):
		self.x = x
		self.y = y 

	def length(self):
		return math.sqrt(self.x**2 + self.y**2)
		
	def __add__(self, other):
		return Point(self.x + other.x, self.y + other.y)

	def __mul__(self, other):
		return Point(self.x * other, self.y * other)
	
	def __neg__(self):
		return Point(-self.x, -self.y)
	
	def __repr__(self):
		return "(%f, %f)" % (self.x, self.y)
	
class AABB:
	def __init__(self, topleft, bottomright, owner = None):
		self.top    = topleft.y
		self.left   = topleft.x
		self.bottom = bottomright.y
		self.right  = bottomright.x
		
		self.owner = owner
		
	def intersectsWith(self, aabb):
		return not (self.left > aabb.right or self.right < aabb.left or self.bottom > aabb.top or self.top < aabb.bottom )
	
	
class CollisionSystem:
	def __init__(self, data_manager):
		self.dataManager = data_manager
	
	def update(self, frametime):
		for actor in self.dataManager.aliveActors:
			if actor.stuck or not actor.doCollisionTest:
				continue
			
			collisions = {}
			for aabb1 in actor.getAABBs():
				collisions[aabb1] = self.testCollisions(aabb1)
				#if actor == self.dataManager.playerActor and len(collisions[aabb1]) > 0:
				#	self.dataManager.playerCollided()
				
				for aabb1, aabb2, direction in collisions[aabb1]:
					#print aabb1.owner.type, aabb2.owner.type
					if aabb2.owner.type  != "Normal" or aabb1.owner.type != "Normal" or aabb1.owner.isPhantom \
					   or direction.length() < 0.1:
						pass
					elif aabb1.owner.speed.x != 0 or aabb1.owner.speed.y != 0:
						#nummoves = 0
						while aabb1.intersectsWith(aabb2):
							#nummoves += 1
							#if nummoves > 21:
							#	print nummoves, direction
							
							p = Point(direction.x * frametime * 0.1, direction.y * frametime * 0.1)
							#print p, "len", direction.length()
							aabb1.owner.move( p )
				
			res = actor.collideWith(collisions)
			
					
	def testCollisions(self, aabb1):
		collisions = []
		
		for actor in self.dataManager.aliveActors:
			if not actor.doCollisionTest: continue
			for aabb2 in actor.getAABBs():
				if aabb1.owner is not aabb2.owner:
					if aabb1.intersectsWith(aabb2):
						collisions.append((aabb1, aabb2, self.findBounceDirection(aabb2, aabb1)))

		return collisions
	
	def findBounceDirection(self, source_aabb, target_aabb):
		intersection = Point(0,0)
		
		maxleft = max(target_aabb.left, source_aabb.left)
		minright = min(target_aabb.right, source_aabb.right)
		intersection.x = minright - maxleft
		
		maxbottom = max(target_aabb.bottom, source_aabb.bottom)
		mintop = min(target_aabb.top, source_aabb.top)
		intersection.y = mintop - maxbottom
		
		if intersection.x < 0 or intersection.y < 0:
			print "wtf", intersection
		
		
		if abs(intersection.x) > abs(intersection.y):
			### They intersected more "along" the x axis, which means it should go up or down
			#return Point( 0, -target_aabb.owner.speed.y )
			if target_aabb.top > source_aabb.top:
				return Point(0, 1 * abs(target_aabb.owner.speed.y))
			else:
				return Point(0, -1 * abs(target_aabb.owner.speed.y))
		else:
			if target_aabb.right > source_aabb.right:
				return Point(1 * abs(target_aabb.owner.speed.x), 0) #came from the right
			else:
				return Point(-1 * abs(target_aabb.owner.speed.x), 0)
			#return Point( -target_aabb.owner.speed.x, 0 )
		
		return Point(-target_aabb.owner.speed.x,-target_aabb.owner.speed.y)
		"""
		
		
		### source = the direction from which the bounce is heading..
		if   target_aabb.right > source_aabb.left  and target_aabb.left < source_aabb.left:
			### Going right
			intersection.x = target_aabb.right - source_aabb.left
		elif target_aabb.left < source_aabb.right  and target_aabb.right > source_aabb.right:
			### Going left
			intersection.x = target_aabb.left - source_aabb.right
		
		if target_aabb.bottom < source_aabb.top    and target_aabb.top > source_aabb.top:
			### Going down
			intersection.y = target_aabb.bottom - source_aabb.top
		elif target_aabb.top > source_aabb.bottom  and target_aabb.bottom < source_aabb.bottom:
			### Going up
			intersection.y = target_aabb.top - source_aabb.bottom
		else:
			print "wtf collision error!!"
			return Point(1,1)
		
		if abs(intersection.x) > abs(intersection.y):
			### They intersected more "along" the x axis, which means it should go up or down
			return Point( 0, -intersection.y )
		else:
			return Point( -intersection.x, 0 )
		"""
		
		