import os
import math

import pygame
#from glFreeType import *
import glFreeType
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
import data
R,G,B,A = 0,1,2,3

from collisionsystem import Point

class Graphics:
	def __init__(self, data_manager, caption):
		self.dataManager = data_manager
		
		pygame.mouse.set_cursor( *pygame.cursors.tri_left )
		pygame.display.set_caption( caption )
		
		self.screen = pygame.display.set_mode((self.dataManager.w, self.dataManager.h), OPENGL|DOUBLEBUF, 32)
		self.resize( (self.dataManager.w, self.dataManager.h ))
		
		#info = pygame.display.Info()
		#print info
		#print info.bitsize
		
		
		self.scale = 1.0
		self.scaleTarget = 1.0
		
		glEnable(GL_TEXTURE_2D)
		glShadeModel(GL_SMOOTH)
		glClearColor(0.0, 0.0, 1.0, 0.0)
		glClearDepth(1.0)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
		glEnable(GL_BLEND)
		glDepthFunc(GL_LEQUAL)
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
		
		
		### Textures
		#print self.dataManager.textures
		p = data.filepath("images")
		#print p
		for (a,b,filenames) in os.walk( p ):
			#print filenames
			for file in filenames:
				if file.find(".png") != -1:
					id = glGenTextures( 1 )
					self.dataManager.spriteManager.spriteIDs[file] = id
					#print "TextureID", id, "mapped to", file
					
					
		for name, id in self.dataManager.spriteManager.spriteIDs.iteritems():
			path = data.filepath( os.path.join("images", name ) )
			#print path
			surface = pygame.image.load( path )
			tdata = pygame.image.tostring(surface, "RGBX", 1)
			glBindTexture(GL_TEXTURE_2D, id)
			glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA8, surface.get_width(), surface.get_height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tdata)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
			
		### Fonts
		self.fonts = {}
		for (name, (filename, size)) in self.dataManager.fonts.iteritems():
			path = data.filepath(filename)
			self.fonts[name] = glFreeType.font_data(path, size)
			
		

	
	def setCaption(self, caption ):
		pygame.display.set_caption( caption )
		
	
	def resize(self, (w, h)):
		self.dataManager.w = w
		self.dataManager.h = h
		
		glViewport(0, 0, w, h)
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		glOrtho(0.0, w,h, 0.0, -1.0, 1.0)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()

	def clearBackground(self):
		### Clear bg
		glColor3d( 1.0, 1.0, 1.0)
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )

	def updateCamera(self):
		cameraspeed = 0.03
		targetpos = self.dataManager.playerActor.pos + self.dataManager.playerActor.speed * 1.5 * self.dataManager.gameSpeed
		targetvec = targetpos + (-self.dataManager.camera)

		if targetvec.length() > 700:
			cameraspeed = 0.05
			targetpos = Point(self.dataManager.playerActor.pos.x, self.dataManager.playerActor.pos.y )
			#print "apa1"

		if targetvec.length() > 1000:
			cameraspeed = 0.3
			#print "apa2"
		
		if not self.dataManager.playerActor.isAlive:
			targetpos = Point(self.dataManager.playerActor.pos.x, self.dataManager.playerActor.pos.y )
			targetpos.y += 50
			
		move = Point(0,0)
		move.x = ( targetpos.x - self.dataManager.camera.x) * cameraspeed
		move.y = ( targetpos.y - self.dataManager.camera.y) * cameraspeed
		self.dataManager.camera.x = self.dataManager.camera.x + move.x
		self.dataManager.camera.y = self.dataManager.camera.y + move.y
		
		scalefactor = 1.0
		if self.dataManager.playerActor.speed.length() > 100:
			scalefactor = 1.0 + 0.2 * self.dataManager.playerActor.speed.length() / 100.0
		self.scale = self.scale + (1.0/(self.dataManager.gameSpeed*scalefactor) - self.scale) * 0.05
		
		### Update water
		for water in self.dataManager.waterActors:
				#diff = Point(self.dataManager.camera.x - water.pos.x, 0)#self.dataManager.playerActor.pos.y - self.pos.y)
				water.move(Point(move.x, 0))
	
		
	def render(self):
		### actual rendering here
		glLoadIdentity()
		#print self.dataManager.playerActor.pos
		### Actors
		current_texture = ""
		for actor in self.dataManager.aliveActors:
			
			### Perform basic culling
			distfromcam = Point(actor.pos.x - self.dataManager.camera.x, actor.pos.y - self.dataManager.camera.y)
			if actor.moveWithCamera and distfromcam.length() > 800 and actor.type != "Water":
				continue
			
			if actor.sprite.image != current_texture:
				current_texture = actor.sprite.image
				id = self.dataManager.spriteManager.spriteIDs[current_texture]
				glBindTexture( GL_TEXTURE_2D, id)
			glColor4d( actor.sprite.color[R], actor.sprite.color[G], actor.sprite.color[B], actor.sprite.color[A])	
			glLoadIdentity()
			### Set 0,0 to be in the center
			scale = 1
			if actor.moveWithCamera:
				scale = (math.sqrt(self.scale))
			#scale = 1.5
			glScaled( scale, scale, 1.0 )
			glTranslated(self.dataManager.w/(2*scale), self.dataManager.h/(2*scale), 0)
			glTranslated(actor.pos.x, -actor.pos.y, 0.0)
			
			if actor.moveWithCamera:
				glTranslated(-self.dataManager.camera.x, self.dataManager.camera.y , 0.0)
			else:
				glTranslated(0.0, 0.0, -0.5)
				
			if actor.type == "Integral":
				glTranslated(0.0, 0.0, 0.5)
			
			#print actor.sprite.frameSize
			halfsize = Point(actor.sprite.frameSize.x / 2, actor.sprite.frameSize.y / 2)
			quad = actor.sprite.getQuad()
#			for p in quad: print p
#			print "--"
			glBegin(GL_QUADS)
			glTexCoord2d( quad[0].x, quad[0].y ); glVertex2d( -halfsize.x, -halfsize.y)
			glTexCoord2d( quad[1].x, quad[1].y ); glVertex2d(  halfsize.x, -halfsize.y)
			glTexCoord2d( quad[2].x, quad[2].y ); glVertex2d(  halfsize.x,  halfsize.y)
			glTexCoord2d( quad[3].x, quad[3].y ); glVertex2d( -halfsize.x,  halfsize.y)
			glEnd()
			
		### Gui
		for element in self.dataManager.guiElements:
			if element.sprite is None:
				continue
			if element.sprite.image != current_texture:
				current_texture = element.sprite.image
				id = self.dataManager.spriteManager.spriteIDs[current_texture]
				glBindTexture( GL_TEXTURE_2D, id)
			
			glLoadIdentity()
			### Set 0,0 to be in the center
			glTranslated(self.dataManager.w/2, self.dataManager.h/2, 0)
			glTranslated(element.pos.x, -element.pos.y, 0.0)

			halfsize = Point(element.sprite.frameSize.x / 2, element.sprite.frameSize.y / 2)
			quad = element.sprite.getQuad()
			
			glBegin(GL_QUADS)
			glTexCoord2d( quad[0].x, quad[0].y ); glVertex2d( -halfsize.x, -halfsize.y)
			glTexCoord2d( quad[1].x, quad[1].y ); glVertex2d(  halfsize.x, -halfsize.y)
			glTexCoord2d( quad[2].x, quad[2].y ); glVertex2d(  halfsize.x,  halfsize.y)
			glTexCoord2d( quad[3].x, quad[3].y ); glVertex2d( -halfsize.x,  halfsize.y)
			glEnd()	
		
		### Text
		glLoadIdentity()
		
		### Set 0,0 to be in the center
		glTranslated(self.dataManager.w/2, self.dataManager.h/2, 0)

		for text in self.dataManager.textElements:
			#print text.text
			if not text.visible:
				continue
			#print text.text, text.pos
			glColor3d( text.color[R], text.color[G], text.color[B])
			self.fonts[text.font].glPrint(text.pos.x, text.pos.y, text.text)
			
			
	def finish(self):
		### Flip buffers
		pygame.display.flip()
		
		