import pygame
from pygame.locals import *

from collisionsystem import Point
import level


class InputGame:
	def __init__(self, data_manager):
		self.dataManager = data_manager

	def capture(self, frametime):
		
		if self.dataManager.playerActor.isAlive:
			pressed = pygame.key.get_pressed()
			if pressed[K_LEFT] and self.dataManager.playerActor.isOnGround:
				self.dataManager.playerActor.acc.x = -2000
			elif pressed[K_LEFT] and not self.dataManager.playerActor.isOnGround:
				self.dataManager.playerActor.speed.x -= 1000 * frametime
			elif pressed[K_RIGHT] and self.dataManager.playerActor.isOnGround:
				self.dataManager.playerActor.acc.x = 2000
			elif pressed[K_RIGHT] and not self.dataManager.playerActor.isOnGround:
				self.dataManager.playerActor.speed.x += 1000 * frametime
		
		events = []	
		for event in pygame.event.get():
			### Quitting
			if event.type == QUIT:
				self.dataManager.keepRunning = False
			elif event.type == KEYDOWN and event.key == K_ESCAPE:
				self.dataManager.currentLevel = level.Level("mainscreen", self.dataManager)
				self.dataManager.currentLevel.start()
				self.dataManager.setInMainScreen(True)

			
			if event.type == KEYDOWN:
				if event.key == K_UP and self.dataManager.playerActor.isOnGround:
					self.dataManager.playerActor.move(Point( 0, 2 ) )
					self.dataManager.playerActor.speed.y = 250
					self.dataManager.playerActor.isOnGround = False
					self.dataManager.playerActor.acc.x = 0
				elif event.key == K_SPACE:
					if self.dataManager.playerActor.isOnGround and not self.dataManager.mathComponent.active:
						self.dataManager.playerActor.move(Point( 0, 2 ) )
						self.dataManager.playerActor.speed.y = 250
						self.dataManager.playerActor.isOnGround = False
						self.dataManager.playerActor.acc.x = 0	
						self.dataManager.mathComponent.activate()
					elif not self.dataManager.mathComponent.active and self.dataManager.playerActor.speed.y > 20:
						self.dataManager.mathComponent.activate()
				elif event.key == K_e:
					self.dataManager.toggleEdit()
				elif event.key == K_y and not self.dataManager.playerActor.isAlive:
					self.dataManager.playerActor.isAlive = True
					self.dataManager.currentLevel.start()
				elif event.key == K_n and not self.dataManager.playerActor.isAlive:
					self.dataManager.currentLevel = level.Level("mainscreen", self.dataManager)
					self.dataManager.currentLevel.start()
					self.dataManager.setInMainScreen(True)
				elif (event.key == K_RETURN or event.key == K_KP_ENTER) and self.dataManager.currentLevel.finished:
					nextlevel = self.dataManager.getNextLevel()
					self.dataManager.currentLevel = level.Level(nextlevel, self.dataManager)
					self.dataManager.currentLevel.start()
					
					if nextlevel == "mainscreen":
						self.dataManager.setInMainScreen(True)
					
					
			if self.dataManager.inLevelEdit:
				
				if event.type == KEYDOWN:
					if event.key == K_s:
						self.dataManager.currentLevel.save()
						print "Level saved as ", self.dataManager.currentLevel.filename
					elif event.key == K_F1:
						self.dataManager.editComponent.spawnActor("tree1")
					elif event.key == K_F2:
						self.dataManager.editComponent.spawnActor("bush")
					elif event.key == K_F3:
						self.dataManager.editComponent.spawnActor("ground_256")
					elif event.key == K_F4:
						self.dataManager.editComponent.spawnActor("ground_64")
					elif event.key == K_F5:
						self.dataManager.editComponent.spawnActor("stone_128")
					elif event.key == K_F6:
						self.dataManager.editComponent.spawnActor("stone_64")
					elif event.key == K_F7:
						self.dataManager.editComponent.spawnActor("flower1")
					elif event.key == K_F8:
						self.dataManager.editComponent.spawnActor("mush1")
					elif event.key == K_F9:
						self.dataManager.editComponent.spawnActor("integral")
					elif event.key == K_F10:
						self.dataManager.editComponent.spawnActor("info")
					else:
						c = (event.unicode).encode('latin-1')
						if c in "123456789" and c != "":
							self.dataManager.editComponent.spawnActor(c)
				elif event.type == MOUSEBUTTONDOWN and event.button == 1:
					self.dataManager.editComponent.placeActor()
				elif event.type == MOUSEBUTTONDOWN and event.button == 3:
					dist = self.dataManager.mousePos + (-self.dataManager.playerActor.pos)
					dist.y -= 50
					self.dataManager.playerActor.move(dist)
			
				x,y = pygame.mouse.get_pos()
				mx = self.dataManager.camera.x - self.dataManager.w/2 + x
				my = self.dataManager.camera.y + self.dataManager.h/2 - y
				self.dataManager.mousePos = Point(mx,my)
			else:
				self.dataManager.textReader.handle_event(event)
	
class TextReader:
	ok_chars = "0123456789-"

	def __init__(self):
		self.reset()
	
	def reset(self):
		self.text = ""
		self.isDone = False
		
	def handle_event(self, event):
		if self.isDone: return
		#print 0
		if event.type == KEYDOWN and event.key == K_BACKSPACE:
			self.text = self.text[:-1]
		elif event.type == KEYDOWN and (event.key == K_RETURN or event.key == K_KP_ENTER):
			self.isDone = True
		elif event.type == KEYDOWN:
			c = (event.unicode).encode('latin-1')
			if self.ok_chars.find(c) > -1 and c != "":
				self.text += c
			else:
				if event.key == K_KP0: self.text += "0"
				elif event.key == K_KP1: self.text += "1"
				elif event.key == K_KP2: self.text += "2"
				elif event.key == K_KP3: self.text += "3"
				elif event.key == K_KP4: self.text += "4"
				elif event.key == K_KP5: self.text += "5"
				elif event.key == K_KP6: self.text += "6"
				elif event.key == K_KP7: self.text += "7"
				elif event.key == K_KP8: self.text += "8"
				elif event.key == K_KP9: self.text += "9"
				elif event.key == K_KP0: self.text += "0"
				elif event.key == K_KP_MINUS: self.text += "-"		
				
										