import os
import pygame


import datamanager
import graphics
import input, inputgame
import physics
import colors 
import sound

import collisionsystem
from collisionsystem import Point
import level
import guisystem
import data

class Game:
	def __init__(self, w, h, caption):
		
		self.dataManager = datamanager.DataManager(w, h)
		self.caption = caption
		
		self.graphics    = graphics.Graphics( self.dataManager, self.caption )
		self.sound       = sound.Sound( self.dataManager )
		self.dataManager.sound = self.sound
		self.input       = input.Input( self.dataManager )
		self.gameInput   = inputgame.InputGame( self.dataManager )
		
		self.physics     = physics.Physics( self.dataManager )
		self.collision   = collisionsystem.CollisionSystem( self.dataManager )
		
		self.gui		 = guisystem.GuiSystem( self.dataManager )


	def startLevel(self, levelname):
		self.dataManager.currentLevel = level.Level(levelname, self.dataManager)
		self.dataManager.currentLevel.start()

		self.dataManager.sound.playSong(data.filepath( os.path.join("audio", "11 - Psychedelique.mp3") ) , 1.0)	
		#self.dataManager.sound.playSong(data.filepath( os.path.join("audio", "01 - Prehistorik Man Amstrad.mp3") ) , 1.1)	
				
	def captureInput(self, frametime):
		if self.dataManager.inMainScreen:
			self.input.capture()
		else:
			self.gameInput.capture( frametime )
	
	def update(self, frametime):
		
		for actor in self.dataManager.aliveActors:
			actor.update( frametime )
				
		self.dataManager.levelCompletedText.visible = False
		self.dataManager.levelCompletedText2.visible = False
		if self.dataManager.currentLevel.finished:
			self.dataManager.levelCompletedText.visible = True
			self.dataManager.levelCompletedText2.visible = True
			return
		
		if self.dataManager.inMainScreen:
			self.dataManager.gameOverText.visible = False
			self.dataManager.gameOverText2.visible = False
			return


			
		
		self.dataManager.gameSpeed = self.dataManager.gameSpeed + (	self.dataManager.targetGameSpeed - self.dataManager.gameSpeed) * 0.05
		#self.dataManager.gameSpeed = 0.3
		frametime *= self.dataManager.gameSpeed
		
		if self.dataManager.playerActor.speed.y < 20:
			self.dataManager.mathComponent.deActivate()
		
		if not self.dataManager.playerActor.isAlive:
			self.dataManager.gameOverText.visible = True
			self.dataManager.gameOverText2.visible = True
			self.dataManager.mathComponent.update( frametime )
			self.dataManager.editComponent.update( frametime )
			self.gui.update( frametime )
			return
		
		self.sound.update( frametime )
		self.physics.update( frametime )
		self.collision.update( frametime)
		
		self.dataManager.mathComponent.update( frametime )
		self.dataManager.editComponent.update( frametime )
#		
#		for actor in self.dataManager.aliveActors:
#			actor.update( frametime )
		
		self.gui.update( frametime )
		self.dataManager.gameOverText.visible = False
		self.dataManager.gameOverText2.visible = False
			
		
		self.dataManager.frameCount += 1
		if not self.dataManager.frameCount % 100:
			self.graphics.setCaption( "%s (fps:%i)" % (self.caption, 1.0/frametime))
		
	def render(self):
		self.graphics.clearBackground()
		
		if not self.dataManager.inMainScreen:
			self.graphics.updateCamera()
			
		self.graphics.render()
		self.graphics.finish()
		
	def keepRunning(self):
		return self.dataManager.keepRunning
	

def main():
	pygame.init()


	w,h = 1024,768
	w,h = 600, 600
	game = Game( w, h, "Math Jumper" )
	game.startLevel( "mainscreen" )
	
	wantedFPS = 40
	
	clock = pygame.time.Clock()
	while game.keepRunning():
		frametime = clock.tick(wantedFPS) * 0.001
		if frametime > 1.0/30:
			frametime = 1.0/30
		frametime

		game.captureInput( frametime)
		game.update( frametime )
		game.render()
		
