
from random import *

from collisionsystem import Point
import colors

class MathComponent:
    def __init__(self, data_manager):
        self.dataManager = data_manager
        self.active = False
        self.ansText = None
        self.exprText = None
        self.mathbg = None
        self.animtimer = 0
        
        self.symbols = []
        self.symbolsGuis = []
        self.symbolsGui = self.dataManager.createGui("CurrentSymbols", Point(0,0), 0.0, None)
        self.symbolsText = self.dataManager.createText("CurrentSymbolsText", "mathFont", colors.white, Point(-400,200), "")
        for s in self.symbols:
            self.symbolsText.text += s + " "
        self.symbolsGui.texts.append(self.symbolsText)
    
    def addSymbol(self, s):
        self.symbols.append(s)
        self.symbolsText.text = ""
        for s in self.symbols:
            self.symbolsText.text += s + " "
            
        w = self.dataManager.w
        h = self.dataManager.h
        x = -w/2 + len(self.symbolsGuis)*64.0 + 32.0
        y = h/2 - 64
        g = self.dataManager.createGui("CurrentSymbols", Point(x,y), 0.0, "star.png")
        g.sprite.setAnimation(s)
        self.symbolsGuis.append(g)
        
    def takeSymbols(self, n):
        ns = self.symbols[:n]
        self.symbols = self.symbols[n:]
        self.symbolsText.text = ""
        for s in self.symbols:
            self.symbolsText.text += s + " "
            
        for g in self.symbolsGuis[:n]:
            self.dataManager.removeGui(g)
        self.symbolsGuis = self.symbolsGuis[n:]    
        for g in self.symbolsGuis:
            g.pos.x -= n*64.0
        return ns
        
    def update(self, frameTime):
        if not self.active: 
        	return
        #print "math active"
        
        self.animtimer += frameTime
        if self.animtimer > 0.6: self.animtimer = 0
        if self.animtimer > 0.3:
            self.ansText.text = "=" + self.dataManager.textReader.text + "|"
        else:
            self.ansText.text = "=" + self.dataManager.textReader.text  
        if self.dataManager.textReader.isDone:
            print "q:" + self.expr,
            print "(q-text:" +  self.exprText.text + ")"
            print "player:" + self.dataManager.textReader.text,
            print "correct:" + str(self.correctAns)
            try:
                player_ans = eval(self.dataManager.textReader.text)
            except:
                player_ans = ""
            
            if player_ans == self.correctAns:
                g = self.dataManager.createGui("RightLabel", Point(0,0), 0.0, "cloud1.png")
                g.static = False
                g.timeLeftToLive = 1.0
                t = self.dataManager.createText("RightText", "mathFont", colors.green, Point(-45,-45), "OK!")
                g.texts.append(t)
                self.dataManager.playerActor.speed.y *= 3
            else:
                g = self.dataManager.createGui("WrongLabel", Point(0,0), 0.0, "cloud1.png")
                g.static = False
                g.timeLeftToLive = 1.0
                t = self.dataManager.createText("RightText", "mathFont", colors.red, Point(-100,-45), "WRONG!")
                g.texts.append(t)
                
            self.active = False
            self.dataManager.targetGameSpeed = 1.0
            self.dataManager.removeText(self.ansText)
            self.dataManager.removeText(self.exprText)
            self.dataManager.removeGui(self.mathbg)
            self.mathbg = None
            self.ansText = None
            self.exprText = None
        return
    
    def activate(self):
        #print "math activate!"
        if self.ansText is not None:
            self.dataManager.removeText(self.ansText)
        if self.exprText is not None:
            self.dataManager.removeText(self.exprText)
        if self.mathbg is not None:
            self.dataManager.removeGui(self.mathbg)
        self.active = False
        
        if len(self.symbols) > 2:
            self.expr = self.gen_math2(len(self.symbols) > 2)
            #while eval(self.expr) > 20:
            #	self.expr = self.gen_math2(len(self.symbols) > 2)
        else:
            return
           
        try:
            self.correctAns = eval(self.expr)
        except:
            return

        self.active = True
        self.dataManager.targetGameSpeed = 0.1
        self.dataManager.gameSpeed = 0.5
        
        self.dataManager.textReader.reset()
        self.mathbg = self.dataManager.createGui("MathBG", Point(0,0), 0.0, "mathbg.png")
        x =  100 + len(self.expr) * 20
        self.mathbg.sprite.size.x = x 
        self.mathbg.sprite.frameSize.x = x 
        self.exprText = self.dataManager.createText("MathQuestion", "mathFont", colors.white, Point(-x/2 + 10,-15), self.expr) 
        self.ansText = self.dataManager.createText("MathAnswer", "mathFont", colors.white, Point(-x/2 + 10,-65), "=")
    
    def deActivate(self):
		self.active = False
		if self.ansText is not None:
			self.dataManager.removeText(self.ansText)
		if self.exprText is not None:
			self.dataManager.removeText(self.exprText)
		if self.mathbg is not None:
			self.dataManager.removeGui(self.mathbg)
		self.mathbg = None
		self.ansText = None
		self.exprText = None
		
		self.dataManager.targetGameSpeed = 1.0
		self.dataManager.gameSpeed = 1.0
    
    def gen_math2(self, hard):
        ops = ['+', '-', '*']
        div_chance = 0.5
        
        if not hard:
            ns = self.takeSymbols(2)
            a = int(ns[0])
            b = int(ns[1])
            op = ""
            if a % b == 0 and random() > div_chance:
                op = "/"
            else:
                op = choice(ops)
            return str(a) + op + str(b)
        ns = self.takeSymbols(3)
        exp = []
       
        r = random()
        if r > 0.67 and int(ns[0] + ns[1]) < 40: # 2,1
            a = int(ns[0] + ns[1])
            b = int(ns[2])
            op = ""
            if a % b == 0 and random() > div_chance:
                op = "/"
                #print a,op,b
            else:
                op = choice(ops)
            exp = str(a) + op + str(b)
        elif r > 0.33: # 1,1,1
            a = int(ns[0])
            b = int(ns[1])
            c = int(ns[2])
            if a % b == 0 and random() > div_chance:
                op1 = "/"
            else:
                op1 = choice(ops)
            if b % c == 0 and random() > div_chance and op1 != "/":
                op2 = "/"
            else:
                op2 = choice(ops)
            exp = str(a) + op1 + str(b) + op2 + str(c)
        else: # 1,2
            a = int(ns[0] + ns[1])
            b = int(ns[2])
            op = choice(ops)
            exp = str(a) + op + str(b)
        
        return exp
    
    