
import level
import colors
from collisionsystem import Point

class MenuComponent:    
    def __init__(self, data_manager):
        self.dataManager = data_manager
        self.levels = ["level01", "level02"]
        self.levelIndex = 0
    
        self.items = ["Level", "StartGame", "Quit"]
        self.itemIndex = 0 
        self.texts = {}
        self.texts["Level"] = self.dataManager.createText("lvltxt", "menuFont", colors.white, Point(-50, 50), self.levels[self.levelIndex])
        self.texts["StartGame"] = self.dataManager.createText("strttxt", "menuFont", colors.grey, Point(-50, 0), "Start Game")
        self.texts["Quit"] = self.dataManager.createText("quittxt", "menuFont", colors.grey, Point(-50, -50), "Quit")
    
    
    def enterMenu(self):
        for k,text in self.texts.iteritems():
            text.visible = True
    
    def leaveMenu(self):
        for k,text in self.texts.iteritems():
            text.visible = False

    def clear(self):
        #self.levels = ["level01", "level02"]
        self.levels = self.dataManager.levels[1:]
        self.levelIndex = 0
    
        self.items = ["Level", "StartGame", "Quit"]
        self.itemIndex = 0 
        self.texts = {}
        self.texts["Level"] = self.dataManager.createText("lvltxt", "menuFont", colors.white, Point(-90, 70), self.levels[self.levelIndex])
        self.texts["StartGame"] = self.dataManager.createText("strttxt", "menuFont", colors.grey, Point(-170, 20), "Start Game")
        self.texts["Quit"] = self.dataManager.createText("quittxt", "menuFont", colors.grey, Point(-50, -30), "Quit")
        if self.dataManager.inMainScreen:
            self.enterMenu()
        else:
            self.leaveMenu()
        
    def activateItem(self):
        if self.items[self.itemIndex] == "Level":
            pass
        elif self.items[self.itemIndex] == "StartGame":
            nextlevel = self.levels[self.levelIndex]
            self.dataManager.currentLevel = level.Level(nextlevel, self.dataManager)
            self.dataManager.currentLevel.start()
            self.dataManager.setInMainScreen(False)
 
            
        elif self.items[self.itemIndex] == "Quit":
            self.dataManager.keepRunning = False
    
    def selectItem(self):
        self.texts[self.items[self.itemIndex]].color = colors.white
    
    def deSelectItem(self):
        self.texts[self.items[self.itemIndex]].color = colors.grey
        
    def down(self):
        self.deSelectItem()
        self.itemIndex += 1
        if self.itemIndex >= len(self.items):
            self.itemIndex = 0
        self.selectItem()
        
    def up(self):
        self.deSelectItem()
        self.itemIndex -= 1
        if self.itemIndex < 0:
            self.itemIndex = len(self.items)-1
        self.selectItem()    
        
    def right(self):
        if self.items[self.itemIndex] != "Level":
            return
        self.levelIndex += 1
        if self.levelIndex >= len(self.levels):
            self.levelIndex = 0
        self.texts[self.items[self.itemIndex]].text = self.levels[self.levelIndex]
        
    def left(self):
        if self.items[self.itemIndex] != "Level":
            return
        self.levelIndex -= 1
        if self.levelIndex < 0:
            self.levelIndex = len(self.levels)-1
        self.texts[self.items[self.itemIndex]].text = self.levels[self.levelIndex]
        
    def update(self, frameTime):
        pass
    