import psyco
psyco.full()

import pygame
from pygame.locals import *

from random import *
import os
import math
from OpenGL.GL import *
from OpenGL.GLU import *
#from vector import *
#from objects import *

# for better code
X,Y = 0,1

def resize((width, height)):
	if height==0:
		height=1
	glViewport(0, 0, width, height)
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	glOrtho(0.0, width, height, 0.0, -1.0, 1.0)
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()

def init():
	glEnable(GL_TEXTURE_2D)
	glShadeModel(GL_SMOOTH)
	glClearColor(0.0, 0.0, 1.0, 0.0)
	glClearDepth(1.0)
	#glEnable(GL_DEPTH_TEST)
	#glEnable(GL_ALPHA_TEST)
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
	glEnable(GL_BLEND)
	
	glDepthFunc(GL_LEQUAL)
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)

def load_textures():
	texture_names = ["cloud1", "cloud2"]
	texture_ids = range(len(texture_names))
	textures = {}
	for name, id in zip(texture_names, texture_ids):
		texturefile = os.path.join('images', name + ".png")
		texture_surface = pygame.image.load(texturefile)
		texture_data = pygame.image.tostring(texture_surface, "RGBX", 1)
		glBindTexture(GL_TEXTURE_2D, id)
		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, texture_surface.get_width(), texture_surface.get_height(), 0,
			  GL_RGBA, GL_UNSIGNED_BYTE, texture_data )
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
		textures[name] = id
	return textures
	
class MainGame:
	

	def main(self):
		# Init pygame
		pygame.init()
		
		#self.screen = pygame.display.set_mode((800, 600), SWSURFACE)
		screen = pygame.display.set_mode((800, 600), OPENGL|DOUBLEBUF)
		#screen = pygame.display.set_mode((800, 600), OPENGL|DOUBLEBUF|self.fullscreen)
		
		pygame.mouse.set_cursor(*pygame.cursors.tri_left )
		pygame.display.set_caption('Test')
		
		#init opengl
		#set size of opengl
		resize((800, 600))
		init()
		
		clock = pygame.time.Clock()
		textures = load_textures()
		self.running = True
		# loop
		while self.running:
			frametime = clock.tick(50)
			events = []	
			for event in pygame.event.get():
				if event.type == QUIT:
					self.running = False

			glColor3d(1.0,1.0,1.0)		
			glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )
			
			position = [0,0]
			
			glBindTexture(GL_TEXTURE_2D, textures["cloud1"])
			glLoadIdentity()
			glTranslated(position[X], position[Y], 0.0)
			glBegin(GL_QUADS)
			glTexCoord2d( 0.0, 1.0 ); glVertex2d(-128.0, -64.0)
			glTexCoord2d( 1.0, 1.0 ); glVertex2d(128.0, -64.0)
			glTexCoord2d( 1.0, 0.0); glVertex2d(128.0, 64.0)
			glTexCoord2d( 0.0, 0.0 ); glVertex2d(-128.0, 64.0)
			glEnd()
						
			pygame.display.flip()

			
def run_game():
	m = MainGame()
	m.main()

def run_test():
	
	import profile
	import pstats

	profile.run( "run_game()", "myStats" )
	p = pstats.Stats( 'myStats' )
	p.strip_dirs()
	p.sort_stats( 'time' )
#	p.sort_stats('cumulative')
	p.print_stats( 30 )
	#p.sort_stats('cumulative').print_stats(10)
	
	"""
	### profiling with hotshot 
	### a bit faster during runtime, a bit slower when calculating the result
	import hotshot, hotshot.stats
	prof = hotshot.Profile("stones.prof")
	prof.runcall(run_game)
	prof.close()
	stats = hotshot.stats.load("stones.prof")
	stats.strip_dirs()
	stats.sort_stats('cumulative', 'time', 'calls')
	stats.print_stats(30)
	"""
if __name__ == '__main__': 
	DO_PROFILING = 1
	if DO_PROFILING:
		run_game()
	else:
		run_test()